This helper function filters optimization results.
filter_results(
results,
run_ids,
optimizer_ids,
which_element,
only_comparable,
keep_empty = FALSE
)A list.
A nested list of optimization results.
Each element corresponds to one optimization run.
It is either list() if the run has been removed or a list
of results for each optimizer.
The results for each optimizer is a list, the output of
apply_optimizer.
A vector of indices. Selects the first layer of results.
A vector of indices. Selects the second layer of results.
A character (vector). Selects the third layer of results.
See documentation of method $results() from Nop object.
Set to TRUE (FALSE, the default) to keep (discard) empty
entries.